@@ -1,3 +1,7 @@
 def message_says_hello(msg):
     """Make sure that the response was friendly"""
     assert msg.payload.get("message") == "hello world"
+
+
+def return_hello(_):
+    return {"hello": "there"}
