@@ -3,12 +3,17 @@
 This is here mainly to make MQTT easier, this will almost defintiely change
 significantly if/when a proper plugin system is implemented!
 """
+import collections.abc
+import dataclasses
 import logging
+from typing import Any, List, Optional
 
 import stevedore
 
 from tavern._core import exceptions
 from tavern._core.dict_util import format_keys
+from tavern._core.pytest.config import TestConfig
+from tavern.request import BaseRequest
 
 logger = logging.getLogger(__name__)
 
@@ -25,7 +30,7 @@ def plugin_load_error(mgr, entry_point, err):
     raise exceptions.PluginLoadError(msg) from err
 
 
-def is_valid_reqresp_plugin(ext):
+def is_valid_reqresp_plugin(ext: Any):
     """Whether this is a valid 'reqresp' plugin
 
     Requires certain functions/variables to be present
@@ -34,7 +39,7 @@ def is_valid_reqresp_plugin(ext):
         Not all of these are required for all request/response types probably
 
     Args:
-        ext (object): class or module plugin object
+        ext: class or module plugin object
 
     Returns:
         bool: Whether the plugin has everything we need to use it
@@ -59,13 +64,16 @@ def is_valid_reqresp_plugin(ext):
     return all(hasattr(ext.plugin, i) for i in required)
 
 
+@dataclasses.dataclass
 class _PluginCache:
     # pylint: disable=inconsistent-return-statements
 
-    def __init__(self):
-        self.plugins = []
+    plugins: List[Any] = dataclasses.field(default_factory=list)
 
-    def __call__(self, config=None):
+    # def __init__(self):
+    #     self.plugins = []
+
+    def __call__(self, config: Optional[TestConfig] = None):
         if not config and not self.plugins:
             raise exceptions.PluginLoadError("No config to load plugins from")
         elif self.plugins:
@@ -131,15 +139,17 @@ class _PluginCache:
 load_plugins = _PluginCache()
 
 
-def get_extra_sessions(test_spec, test_block_config):
+def get_extra_sessions(
+    test_spec: collections.abc.Mapping, test_block_config: TestConfig
+) -> dict:
     """Get extra 'sessions' for any extra test types
 
     Args:
-        test_spec (dict): Spec for the test block
-        test_block_config (dict): available config for test
+        test_spec: Spec for the test block
+        test_block_config: available config for test
 
     Returns:
-        dict: mapping of name: session. Session should be a context manager.
+        mapping of name to session. Session should be a context manager.
     """
 
     sessions = {}
@@ -161,18 +171,22 @@ def get_extra_sessions(test_spec, test_block_config):
     return sessions
 
 
-def get_request_type(stage, test_block_config, sessions):
+def get_request_type(
+    stage: collections.abc.Mapping,
+    test_block_config: TestConfig,
+    sessions: collections.abc.Mapping,
+) -> BaseRequest:
     """Get the request object for this stage
 
     there can only be one
 
     Args:
-        stage (dict): spec for this stage
-        test_block_config (dict): variables for this test run
-        sessions (dict): all available sessions
+        stage: spec for this stage
+        test_block_config: variables for this test run
+        sessions: all available sessions
 
     Returns:
-        BaseRequest: request object with a run() method
+        request object with a run() method
 
     Raises:
         exceptions.DuplicateKeysError: More than one kind of request specified
@@ -217,16 +231,18 @@ class ResponseVerifier(dict):
     plugin_name: str
 
 
-def _foreach_response(stage, test_block_config, action):
+def _foreach_response(
+    stage: collections.abc.Mapping, test_block_config: TestConfig, action
+):
     """Do something for each response
 
     Args:
-        stage (dict): Stage of test
-        test_block_config (dict): Config for test
+        stage: Stage of test
+        test_block_config: Config for test
         action ((p: {plugin, name}, response_block: dict) -> Any): function that takes (plugin, response block)
 
     Returns:
-        dict: mapping of plugin name: list of expected (normally length 1)
+        mapping of plugin name to list of expected (normally length 1)
     """
 
     plugins = load_plugins(test_block_config)
@@ -241,7 +257,11 @@ def _foreach_response(stage, test_block_config, action):
     return retvals
 
 
-def get_expected(stage, test_block_config, sessions):
+def get_expected(
+    stage: collections.abc.Mapping,
+    test_block_config: TestConfig,
+    sessions: collections.abc.Mapping,
+):
     """Get expected responses for each type of request
 
     Though only 1 request can be made, it can cause multiple responses.
@@ -251,12 +271,12 @@ def get_expected(stage, test_block_config, sessions):
     BEFORE running the request.
 
     Args:
-        stage (dict): test stage
-        test_block_config (dict): available configuration for this test
-        sessions (dict): all available sessions
+        stage: test stage
+        test_block_config: available configuration for this test
+        sessions: all available sessions
 
     Returns:
-        dict: mapping of request type: expected response dict
+        mapping of request type to expected response dict
     """
 
     def action(p, response_block):
@@ -273,17 +293,22 @@ def get_expected(stage, test_block_config, sessions):
     return _foreach_response(stage, test_block_config, action)
 
 
-def get_verifiers(stage, test_block_config, sessions, expected):
+def get_verifiers(
+    stage: collections.abc.Mapping,
+    test_block_config: TestConfig,
+    sessions: collections.abc.Mapping,
+    expected: collections.abc.Mapping,
+):
     """Get one or more response validators for this stage
 
     Args:
-        stage (dict): spec for this stage
-        test_block_config (dict): variables for this test run
-        sessions (dict): all available sessions
-        expected (dict): expected responses for this stage
+        stage: spec for this stage
+        test_block_config: variables for this test run
+        sessions: all available sessions
+        expected: expected responses for this stage
 
     Returns:
-        BaseResponse: response validator object with a verify(response) method
+        response validator object with a verify(response) method
     """
 
     def action(p, _):  # pylint: disable=unused-argument
